import { Component } from '@angular/core';
import { FormGroup, FormControl } from '@angular/forms';
import { AuthService } from '../services/auth.service';
import { ToastService } from '../services/toast.service';
import { UserService } from '../services/user.service';
import { Unsubscriber } from '../components/common/unsubscriber';
import { NavController } from '@ionic/angular';
import { takeUntil, skip } from 'rxjs/operators';
import { TokenService } from '../services/token.service';
import { PatientService } from '../services/patient.service';
import { StateService } from '../services/state.service';
import { NotificationService } from '../services/notification.service';
import {ActivatedRoute, ParamMap, Router, NavigationExtras} from '@angular/router';

// @Unsubscriber()
@Component({
  selector: 'app-auth',
  templateUrl: './auth.page.html',
  styleUrls: ['./auth.page.scss'],
})
export class AuthPage extends Unsubscriber {

  previousUser;
  loginForm: FormGroup = new FormGroup({
    email: new FormControl(''),
    password: new FormControl(''),
  });

  private routeSub;
  private returnUrl;
  public isDoneSubmitting;
  public isDoneLoading;

  constructor(
      private authService: AuthService,
      private nav: NavController,
      private toast: ToastService,
      private userService: UserService,
      private tokenService: TokenService,
      private patientService: PatientService,
      private stateService: StateService,
      private notificationService: NotificationService,
      private route: ActivatedRoute,
      private router: Router,
  ) {
    super();

    this.route.queryParams.subscribe(params => {
      this.returnUrl = params.returnUrl;
    });
  }

  ionViewWillEnter() {
    this.isDoneLoading = false;
    this.isDoneSubmitting = true;
    this.userService.previousUser.pipe(takeUntil(this._unsubscriber)).subscribe((user) => {
      this.previousUser = user;
      if (this.previousUser && this.previousUser.email) {
        this.loginForm.controls.email.setValue(this.previousUser.email);
      } else {
        this.loginForm.controls.email.setValue('');
      }
    });
    this.isDoneLoading = true;
    this.autoLogin();
  }

  onLoginFormSubmit() {
    this.isDoneSubmitting = false;
    const email = this.loginForm.get('email').value;
    const password = this.loginForm.get('password').value;
    const credentials = {email, password};
    this.authService.login(credentials).pipe(takeUntil(this._unsubscriber)).subscribe((authResult) => {
          if (authResult) {
            this.notificationService.registerNotifications();
            this.patientService.patients.pipe(skip(1)).pipe(takeUntil(this._unsubscriber)).subscribe(
                () => {
                  this.routeAfterLogin();
                });
            this.patientService.getPatients();
          }
        },
        (error) => {
          this.isDoneSubmitting = true;
          if (error.statusCode === 400 || error.statusCode === 403) {
            this.toast.presentToast(error.description);
          }
        });
  }

  public async onForgotPassword() {
    this.nav.navigateForward(`/forgot-password`);
  }

  private async autoLogin() {
    this.isDoneLoading = true;
    const token = this.tokenService.token;
    if (token) {
      if (this.tokenService.isTokenExpired(token)) {
        console.log('token is expired');
        return;
      }

      this.authService.autoLogin().pipe(takeUntil(this._unsubscriber)).subscribe((authResult) => {
            if (authResult) {
              this.notificationService.registerNotifications();
              this.patientService.patients.pipe(skip(1)).pipe(takeUntil(this._unsubscriber)).subscribe(
                  () => {
                    this.routeAfterLogin();
                  });
              this.patientService.getPatients();
            }
          },
          (error) => {
            if (error.statusCode === 400 || error.statusCode === 403) {
              this.toast.presentToast(error.description);
            }
          });
    } else {
      this.userService.checkForPreviousUser();
    }
  }

  public async clearpreviousUser() {
    this.authService.invalidate();
  }

  private routeAfterLogin() {
    if (this.returnUrl) {
      this.nav.navigateRoot(this.returnUrl);
    } else {
      if (!this.userService.isDoctor) {
        this.patientService.patients.subscribe((res) => {
          this.patientService.setCurrentPatient(res[0]);
          this.nav.navigateRoot(`/timeline`);
        });
      } else {
        const route = this.userService.isDoctor ? ['/patients'] : ['/timeline'];
        this.nav.navigateRoot(route);
      }
    }
  }
}
