import { Component, OnInit } from '@angular/core';
import { Unsubscriber } from '../common/unsubscriber';
import * as moment from 'moment';
import { UserService } from 'src/app/services/user.service';
import { NavigationExtras } from '@angular/router';
import { NavController } from '@ionic/angular';
import {CommunicationService} from '../../services/communication.service';
import { NotificationService } from 'src/app/services/notification.service';

@Component({
  selector: 'app-communication',
  templateUrl: './communication.component.html',
  styleUrls: ['./communication.component.scss'],
})
export class CommunicationComponent extends Unsubscriber implements OnInit {
  public pageTitle;
  public data = [];
  public subscribedChannels = [];
  private twilioUserToken;
  private twilioClient;
  public isDoneLoading;
  constructor(
    public userService: UserService,
    private nav: NavController,
    private communicationService: CommunicationService,
    public notificationService: NotificationService
  ) {
    super();
   }

  ngOnInit() {
    this.isDoneLoading = false;
    this.pageTitle = this.userService.isDoctor ? 'Communication Center' : 'Messages';

    const notifications = this.notificationService.chatMessages;

    notifications.getValue().forEach(chatMessage => {
      if (typeof chatMessage.id !== 'undefined') {
        this.notificationService.deleteSingleNotification(chatMessage.id, 'CHAT_MESSAGE');
      }
    });
  }

  ionViewWillEnter() {
    this.initializeTwilio();
  }

  getFilteredData(ev: any) {
    const val = ev.target.value;

    if (val && val.trim() !== '') {
      this.data = this.data.filter((data) => {
        const fullName = data.first_name + ' ' + data.last_name;
        return (fullName.toLowerCase().indexOf(val.toLowerCase()) > -1);
      });
    }
  }

  goToChannelMessages(data) {
    if (data) {
      const navigationExtras: NavigationExtras = {
          queryParams: {
            id: data.id,
            first_name: data.first_name,
            last_name: data.last_name,
            picture: data.picture,
            date: data.date,
            message: data.message,
            withHistory: true,
            channel_sid: data.sid,
            channel_name: data.friendlyName
          }
      };
      this.nav.navigateForward([`/messages`], navigationExtras);
    } else {
      this.nav.navigateForward([`/messages`]);
    }
  }

  goToRecipients() {
    const navigationExtras: NavigationExtras = {
        queryParams: {
            isVideo: true
        }
    };

    this.nav.navigateForward([`/recipients`], navigationExtras);
  }

  // Initilize twilio client
  initializeTwilio() {
    this.communicationService.getUserMessageToken().subscribe(async res => {

      this.twilioUserToken = res.token;
      const Chat = require('twilio-chat');

      // Create Initalize the Chat Client
      this.twilioClient = new Chat.Client(this.twilioUserToken);

      await this.acceptInviteToChannel();
      await this.getSubscribedChannels();
    });
  }

  // Accept Any Pending Invites
  acceptInviteToChannel() {
    // Listen for new invitations to your Client
    this.twilioClient.on('channelInvited', channel => {
      // Join the channel that you were invited to
      channel.join().then(() => {this.getSubscribedChannels(); });
    });
  }

  // Get Subscribed Channels from Twilio
  getSubscribedChannels() {
      const subscribedChannels = [];

      // let subscribedChannel = client.getSubscribedChannels();
      this.twilioClient.getSubscribedChannels().then(paginator => {

        for (let i = 0; i < paginator.items.length; i++) {
          const channel = paginator.items[i];

          let last_message = '';
          let first_name = '';

          // listener for new messages
          channel.on('messageAdded', message => {
            channel.getMessages().then(messages => {
              const totalMessages = messages.items.length;
              for (i = 0; i < totalMessages; i++) {
                const addedMessage = JSON.parse(messages.items[i].body);
                last_message =   addedMessage.message;
                first_name = addedMessage.firstName;
                channel.channelLastDate = this.convertDate(addedMessage.updatedAt);
              }
            }).then(() => {
              channel.channelLastMessage = first_name + ': ' + last_message;
            });
          });

          channel.getMessages().then(messages => {
            const totalMessages = messages.items.length;
            for (i = 0; i < totalMessages; i++) {
              const message = JSON.parse(messages.items[i].body);
              last_message =   message.message;
              first_name = message.firstName;
              channel.channelLastDate = this.convertDate(message.updatedAt);
            }
          }).then(() => {
            channel.channelLastMessage = first_name + ': ' + last_message;
          });

          subscribedChannels.push(channel);
          this.sortByDesc(subscribedChannels, 'dateUpdated');
          this.isDoneLoading = true;
        }
      });

      this.subscribedChannels = subscribedChannels;

  }

  // Delete Current User Channels
  deleteAllUserChannels() {
    this.isDoneLoading = false;
    this.twilioClient.getSubscribedChannels().then((paginator) => {
      paginator.items.foreach((channel) => {
        channel.delete().then((c) => {
          console.log('Deleted channel: ' + c.sid);
          this.isDoneLoading = true;
        });
      });
    });
  }

  deleteUserChannel(d) {
    d.delete().then(() => {
      this.getSubscribedChannels();
    });
  }

  convertDate(date) {
    date = new Date(date);
    return date.toLocaleString();
  }

  sortByDesc(array, key) {
    return array.sort((a, b) => {
      const x = a[key]; const y = b[key];
      return ((x > y) ? -1 : ((x < y) ? 1 : 0));
    });
  }
}
