import { Component, OnInit, OnDestroy } from '@angular/core';
import { ActivatedRoute, ParamMap } from '@angular/router';

import { GoalService } from 'src/app/services/goal.service';
import { Observable } from 'rxjs';
import { switchMap, map } from 'rxjs/operators';
import { IGoal } from 'src/app/models/server/goal';
import { UserService } from 'src/app/services/user.service';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { PopoverController, AlertController, NavController } from '@ionic/angular';
import { IHistory } from 'src/app/models/server/history';
import { PopoverMenuComponent } from '../common/popover-menu/popover-menu.component';
import { OverlayEventDetail } from '@ionic/core';
import { ToastService } from '../../services/toast.service';

@Component({
  selector: 'app-goal',
  templateUrl: './goal.component.html',
  styleUrls: ['./goal.component.scss']
})
export class GoalComponent implements OnInit, OnDestroy {
  public goal$: Observable<IGoal | {}>;
  public pageTitle = 'Goal';
  private goalId: string;
  public progressForm: FormGroup;
  public history$: Observable<IHistory[]>;
  public completeDate$;
  public showNoteInput = false;
  public patientId: string;
  public isComplete = false;
  private goalSubscription;
  public goal;
  public i = Array;
  public confettiCount = 149;
  public triggerCelebration = false;
  public showSaveButton: boolean;
  public isPercentageChanged = false;
  public isNoteChanged = false;

  constructor(
    private route: ActivatedRoute,
    private goalService: GoalService,
    public userService: UserService,
    private fb: FormBuilder,
    public popoverController: PopoverController,
    private alertController: AlertController,
    private nav: NavController,
    private toast: ToastService
  ) { }

  ngOnInit() {
    this.progressForm = this.fb.group({
      percentage: ['', Validators.required],
      notes: [null]
    });

    this.goal$ = this.goalService.goal;
    this.showSaveButton = false;

    this.goalSubscription = this.route.paramMap.pipe(
      switchMap((params: ParamMap) => {
        this.patientId = params.get('id');
        this.goalId = params.get('goal_id');
        this.getGoal();
        return this.goal$;
      })
    )
    .subscribe((goal: IGoal) => {
      this.goal = goal;
      this.isComplete = goal.current_percentage === '100' ? true : false;
      if (this.isComplete) {
        this.completeDate$ = this.goalService.goalHistory.pipe(
          map((history) => {
            return history.filter(update => update.percentage === 100)[0];
          })
        );
      } else {
        this.history$ = this.goalService.goalHistory;
        this.progressForm.get('percentage').setValue(goal.current_percentage);
      }
      this.getHistory();
    });
  }

  ngOnDestroy(): void {
    if (this.goalSubscription) {
      this.goalSubscription.unsubscribe();
    }
  }

  getGoal() {
    this.goalService.getGoal(this.goalId);
  }

  getHistory() {
    this.goalService.getGoalHistory(this.goalId);
  }

  saveProgress() {
    if (this.progressForm.value.notes  === '' || this.progressForm.value.notes  === null) {
      this.progressForm.value.notes = 'Progress Updated!';
    }
    this.goalService.updateGoalProgress(this.goalId, this.progressForm.value).subscribe((val) => {
      this.progressForm.get('notes').reset();
      this.showNoteInput = false;
      this.getGoal();
      this.getHistory();
      if (val.percentage === 100) {
        this.triggerCelebration = true;
      }
      this.toast.presentToast('Awesome! See the history below.');
    });
  }

  percentageChanged(new_percentage) {
    this.isPercentageChanged = this.goal.current_percentage !== new_percentage ? true : false;
    this.showSaveButton = this.isPercentageChanged || this.isNoteChanged;
  }

  noteChanged(new_note) {
    const checkSpace = /^\s+$/.test(new_note.detail.value);
    this.isNoteChanged = this.goal.notes !== new_note && new_note.detail.value.length > 0 && !checkSpace ? true : false;
    this.showSaveButton = this.isPercentageChanged || this.isNoteChanged;
  }

  async showMenu(ev: any) {
    const popover = await this.popoverController.create({
      component: PopoverMenuComponent,
      componentProps: {
        menu: [
        {text: 'Edit', icon: 'create'},
        {text: 'Delete', icon: 'close-circle-outline'}]},
      event: ev,
      translucent: true
    });
    popover.onDidDismiss().then((detail: OverlayEventDetail) => {
      if (detail !== null) {
        if (detail.data === 'Edit') {
          this.nav.navigateForward([`/patient/${this.patientId}/goals/${this.goalId}/edit`]);
        }
        if (detail.data === 'Delete') {
          this.presentAlertConfirm();
        }
      }
   });

    return await popover.present();
  }

  async presentAlertConfirm() {
    const alert = await this.alertController.create({
      header: 'Alert!',
      message: 'Are you sure you want to delete this goal?',
      buttons: [
        {
          text: 'No',
          role: 'cancel',
          cssClass: 'secondary'
        }, {
          text: 'Yes',
          cssClass: 'dark',
          handler: () => {
            this.goalService.deleteGoal(this.goalId).subscribe((val) => {
              this.nav.navigateForward([`/patient/${this.patientId}/goals`]);
            });
          }
        }
      ]
    });

    await alert.present();
  }

  navigateToGoals() {
    this.nav.navigateForward([`/patient/${this.patientId}/goals`]);
  }
}
