import { Component, OnInit } from '@angular/core';
import { UserService } from 'src/app/services/user.service';
import { tap, map } from 'rxjs/operators';
import { StateService } from 'src/app/services/state.service';
import { ReportsService } from 'src/app/services/reports.service';
import * as moment from 'moment';
import { Papa } from 'ngx-papaparse';
import { Platform } from '@ionic/angular';
import { File } from '@ionic-native/file/ngx';
import { SocialSharing } from '@ionic-native/social-sharing/ngx';
import * as _ from 'lodash';
import { TherapistWeekOverWeekEntry } from 'src/app/models/server/WeekOverWeekReport';

@Component({
  selector: 'app-leaderboard',
  templateUrl: './leaderboard.component.html',
  styleUrls: ['./leaderboard.component.scss'],
})

export class LeaderboardComponent implements OnInit {
  public pageTitle = 'Leaderboard';
  public platform;
  public data: TherapistWeekOverWeekEntry[];
  public startDate;
  public endDate;
  public start;
  public end;
  public prevStart;
  public prevEnd;
  public totalSessions;
  public totalComment;
  public totalGuardianActivity;
  public percentageSessions;
  public percentageComment;
  public percentageGuardianActivity;

  constructor(
    public userService: UserService,
    public stateService: StateService,
    public reportsService: ReportsService,
    public papa: Papa,
    private plt: Platform,
    private file: File,
    private socialSharing: SocialSharing

  ) { }

  ngOnInit() {
    this.platform = this.stateService.getPlatform();
    this.getCurrentWeek();
  }

  getCurrentWeek() {
    const ourDate = new Date();
    const week = this.setDateValues(ourDate);
    this.start = week[0];
    this.end = week[6];
    this.startDate = moment(this.start).format('MMM DD, YYYY');
    this.endDate = moment(this.end).format('MMM DD, YYYY');
    this.getTherapistWeekOverWeek(this.start, this.end);
  }

  getPreviousWeek() {
    const pastDate = moment(moment(this.start).subtract(7, 'days')).toDate();
    const week = this.setDateValues(pastDate);
    this.start = week[0];
    this.end = week[6];
    this.startDate = moment(this.start).format('MMM DD, YYYY');
    this.endDate = moment(this.end).format('MMM DD, YYYY');
    this.getTherapistWeekOverWeek(this.start, this.end);
  }

  getNextWeek() {
    const nextDate = moment(moment(this.start).add(7, 'days')).toDate();
    const week = this.setDateValues(nextDate);
    this.start = week[0];
    this.end = week[6];
    this.startDate = moment(this.start).format('MMM DD, YYYY');
    this.endDate = moment(this.end).format('MMM DD, YYYY');
    this.getTherapistWeekOverWeek(this.start, this.end);
  }

  setDateValues(date) {
    const week = [];

    for (let i = 1; i <= 7; i++) {
      const first = date.getDate() - date.getDay() + i;
      const day = new Date(date.setDate(first)).toISOString();
      week.push(day);
    }

    return week;
  }

  countTotals() {
    let totalSessions = 0;
    let totalGuardianActivity = 0;
    let totalComment = 0;
    let totalSessionsPrev = 0;
    let totalGuardianActivityPrev = 0;
    let totalCommentPrev = 0;

    for (const i of this.data) {
      totalSessions += Number(i.currentSessionsCount);
      totalComment += Number(i.previousCommentCount);
      totalGuardianActivity += Number(i.currentGuardianActivityCount);
      totalSessionsPrev += Number(i.previousSessionsCount);
      totalCommentPrev += Number(i.previousCommentCount);
      totalGuardianActivityPrev += Number(i.previousGuardianActivityCount);
    }

    this.totalSessions = totalSessions;
    this.totalComment = totalComment;
    this.totalGuardianActivity = totalGuardianActivity;

    this.percentageSessions = this.getPercentage(totalSessions, totalSessionsPrev);
    this.percentageComment = this.getPercentage(totalComment, totalCommentPrev);
    this.percentageGuardianActivity = this.getPercentage(totalGuardianActivity, totalGuardianActivityPrev);
  }

  getPercentage(curr, prev) {
    let percent = curr > 0 && prev > 0 ? Math.floor(curr / prev * 100) : 0;
    if (percent > 100) {
      percent = 100;
    }

    return percent;
  }

  getTherapistWeekOverWeek(start, end) {
    const pastDate = moment(moment(start).subtract(7, 'days')).toDate();
    const week = this.setDateValues(pastDate);
    this.prevStart = week[0];
    this.prevEnd = week[6];
    this.reportsService.getTherapistWeekOverWeek(this.prevStart, end)
      .subscribe((currWeek) => {
        const fullData = currWeek.map(entry => {
          return new TherapistWeekOverWeekEntry().fromJSON(entry);
        });
        this.data = this.isDescending(fullData, 'currentSessionsCount');
        this.countTotals();
      });
  }

  sort(ev) {
    const column = ev.srcElement.title;
    if (ev.srcElement.name === 'md-arrow-dropup') {
      ev.srcElement.name = 'md-arrow-dropdown';
      this.isAscending(this.data, column);
    } else if (ev.srcElement.name === 'md-arrow-dropdown') {
      ev.srcElement.name = 'md-arrow-dropup';
      this.isDescending(this.data, column);
    }
  }

  isAscending(arr, prop) {
    return arr.sort((a, b) => (a[prop] > b[prop]) ? 1 : -1);
  }

  isDescending(arr, prop) {
    return arr.sort((a, b) => (a[prop] > b[prop]) ? -1 : 1);
  }

  exportCSV() {
    const toCSV = [];
    let picked;

    for (const i of this.data) {
      picked = _.pick(i, ['therapistName', 'lastLoginStr',
      'currentSessionsCount', 'previousSessionsCount',
      'currentCommentCount', 'previousCommentCount',
      'currentViewCount', 'previousViewCount',
      'currentGuardianActivityCount', 'previousGuardianActivityCount']);
      toCSV.push(picked);
    }

    const csv = this.papa.unparse({
      data: toCSV
    });
    const fileName = 'leaderboard-'
    + moment(this.start).format('MMDDYYYY') + '-'
    + moment(this.end).format('MMDDYYYY') + '.csv';

    if (this.plt.is('cordova')) {
      this.file.writeFile(this.file.dataDirectory, fileName, csv, { replace: true }).then(res => {
        this.socialSharing.share(null, null, res.nativeURL, null);
      });
    } else {
      const blob = new Blob([csv]);
      const a = window.document.createElement('a');
      a.href = window.URL.createObjectURL(blob);
      a.download = fileName;
      document.body.appendChild(a);
      a.click();
      document.body.removeChild(a);
    }

  }

}
