import { Component, OnInit } from '@angular/core';
import { Unsubscriber } from '../common/unsubscriber';
import { ViewChild } from '@angular/core';
import { IonSlides, IonContent, NavController } from '@ionic/angular';
import { StateService } from '../../services/state.service';
import { FormGroup, Validators, FormBuilder } from '@angular/forms';
import { ToastService } from 'src/app/services/toast.service';
import { map, takeUntil } from 'rxjs/operators';
import { InvitationService } from 'src/app/services/invitation.service';
import { UserService } from 'src/app/services/user.service';

@Component({
  selector: 'app-signup',
  templateUrl: './signup.component.html',
  styleUrls: ['./signup.component.scss'],
})
export class SignUpComponent extends Unsubscriber implements OnInit {
  public therapyTypes;
  @ViewChild(IonSlides) slides: IonSlides;
  @ViewChild(IonContent) content: IonContent;
  private platform;
  public downloadLink;
  public therapistSignRegisterForm: FormGroup;
  public parentSignRegisterForm: FormGroup;
  public childForm: FormGroup;
  public addOtherCaregiversForm: FormGroup;
  public addTherapyProvider: FormGroup;
  public addAdditionalOrganizations: FormGroup;
  public invitation;
  public userToken;
  public patientCount: number;
  public organizationId;
  public isDoctor;
  public user;
  public picture;
  public fullName;
  public enableAgreeButton;
  public todayDate: string = new Date().toISOString();

  public actionSheetOptions = {
    header: 'Select Type',
    cssClass: 'custom-action-sheet'
  };

  constructor(
    private stateService: StateService,
    private fb: FormBuilder,
    private toastService: ToastService,
    private invitationService: InvitationService,
    private userService: UserService,
    private nav: NavController,
  ) {
    super();
  }

  ngOnInit() {
    const url_string = window.location.href;
    const url = new URL(url_string);
    this.userToken = String(url.searchParams.get('userToken'));

    this.getValidators();

    setTimeout(() => {
      this.invitationService.getUserInvitation(this.userToken).pipe(
        takeUntil(this._unsubscriber)
        ).subscribe((res) => {
        this.patientCount = res.patientCount;
        this.organizationId = res.organizationId;
        this.isDoctor = res.isDoctor;
        this.user = res.user;
        this.picture = res.user.picture;
        this.fullName = res.user.first_name + ' ' + res.user.last_name;
        this.enableAgreeButton = true;
      }, (err) => {
        this.toastService.presentToastWithClose('Invitation already used. Please contact support.');
        console.log(err);
      });
    }, 1000);

    this.slides.lockSwipes(true);

    this.therapyTypes = this.invitationService.getSpecialties(this.userToken).pipe(
      map(val => val.specialties)
    );
  }

  getValidationError(form: FormGroup, propertyName: string): boolean {
    try {
      const property = form.get(propertyName);
      return !!property && !!property.dirty && !!property.errors;
    } catch {
      return false;
    }
  }

  next(skipStep: boolean) {
    let data;

    this.slides.getActiveIndex().then(index => {
      if (!this.isDoctor) {
        switch (index) {
          case 0:
            this.goToNextSlide();
            break;
          case 1:
            if (this.parentSignRegisterForm.value.confirmPasswordParent !== this.parentSignRegisterForm.value.passwordParent) {
              this.toastService.presentToast('Password Mismatch');
            } else {
              if (this.patientCount > 0) {
                data = {
                  userToken: this.userToken,
                  newPassword: this.parentSignRegisterForm.value.confirmPasswordParent,
                  phoneNumber: this.parentSignRegisterForm.value.phoneNumber
                };

                this.finalizeUser(data).then(() => {
                  this.goToNextSlide(3);
                }, () => {});
              } else {
                this.goToNextSlide();
              }
            }
            break;
          case 2:
            data = {
              userToken: this.userToken,
              newPassword: this.parentSignRegisterForm.value.confirmPasswordParent,
              phoneNumber: this.parentSignRegisterForm.value.phoneNumber,
              patients: [{
                organizationId: this.organizationId,
                firstName: this.childForm.value.childFirstName,
                lastName: this.childForm.value.childLastName,
                dateOfBirth: this.childForm.value.childDateOfBirth,
                diagnosis: this.childForm.value.conditionDiagnosis
              }]
            };
            this.finalizeUser(data).then(() => {
              this.goToNextSlide();
            }, () => {});
            break;
          case 3:
            if (skipStep) {
              this.goToNextSlide();
              return;
            }
            data = {
              userToken: this.userToken,
              email: this.addOtherCaregiversForm.value.email,
              firstName: this.addOtherCaregiversForm.value.firstName,
              lastName: this.addOtherCaregiversForm.value.lastName,
              organizationId: this.organizationId
            };

            this.invitationService.inviteUser(data).subscribe(() => {
              this.goToNextSlide();
            }, (err) => {
              this.toastService.presentToastTopWithClose(`Error inviting additional caregiver. Please try again or contact support.`);
              console.log(JSON.stringify(err));
            });
            break;
          case 4:
            if (skipStep) {
              this.goToNextSlide();
              return;
            }
            this.invitationService.addNewOrganization(this.addTherapyProvider.value, this.userToken).subscribe((val) => {
              this.goToNextSlide();
            });
            break;
          default: break;
        }
      } else {
        switch (index) {
          case 0:
            this.goToNextSlide();
            break;
          case 1:
            if (this.therapistSignRegisterForm.value.confirmPasswordDoctor !== this.therapistSignRegisterForm.value.passwordDoctor) {
              this.toastService.presentToast('Password Mismatch');
            } else {
              data = {
                userToken: this.userToken,
                specialtyId: this.therapistSignRegisterForm.value.specialtyId,
                phoneNumber: this.therapistSignRegisterForm.value.phoneNumber,
                newPassword: this.therapistSignRegisterForm.value.confirmPasswordDoctor
              };

              this.finalizeUser(data).then(() => {
                this.goToNextSlide();
              }, () => {});
            }
            break;
          default: break;
        }
      }
    });

    this.content.scrollToTop(0);
  }

  goToNextSlide(step: number = -1) {
    this.slides.lockSwipes(false);
    if (step > -1) {
      this.slides.slideTo(step);
    } else {
      this.slides.slideNext();
    }
    this.slides.lockSwipes(true);
  }

  finalizeUser(data): Promise<null> {
    return new Promise((resolve, reject) => {
      this.invitationService.finalizeInvitation(data).subscribe(() => {
        resolve();
      }, (err) => {
        this.toastService.presentToastTopWithClose(`Error updating account. Please try again or contact support.`);
        console.log(JSON.stringify(err));
        reject();
      });
    });
  }

  getEnd(e) {
    this.slides.pager = false;
  }

  getMobileOperatingSystem() {
    const userAgent = navigator.userAgent || navigator.vendor;

    if (/android/i.test(userAgent)) {
      return 'Android';
    }

    // iOS detection from: http://stackoverflow.com/a/9039885/177710
    if (/iPad|iPhone|iPod/.test(userAgent)) {
      return 'iOS';
    }

    return 'unknown';
  }

  public async download() {
    this.platform = (await this.stateService.getDeviceInfo() || {}).platform;

    const deviceType = this.getMobileOperatingSystem();

    if (deviceType === 'iOS') {
      window.open('https://apps.apple.com/us/app/therawe-connect/id1276294697');
    } else if (deviceType === 'Android') {
      window.open('https://play.google.com/store/apps/details?id=com.theraweconnect.TheraWe');
    } else {
      this.nav.navigateRoot(`/`);
    }
  }

  public async openWeb() {
    this.nav.navigateRoot(`/`);
  }

  getValidators() {
    this.therapistSignRegisterForm = this.fb.group({
      specialtyId: ['', Validators.required],
      phoneNumber: ['',
        Validators.compose([Validators.maxLength(70),
          Validators.pattern('^[0-9()-]*$'),
        Validators.required])],
      passwordDoctor: ['', Validators.required],
      confirmPasswordDoctor: ['', Validators.required]
    });

    this.parentSignRegisterForm = this.fb.group({
      phoneNumber: ['',
        Validators.compose([Validators.maxLength(70),
        Validators.pattern('^[0-9()-]*$'),
        Validators.required])],
      passwordParent: ['', Validators.required],
      confirmPasswordParent: ['', Validators.required]
    });

    this.childForm = this.fb.group({
      childFirstName: ['', Validators.required],
      childLastName: ['', Validators.required],
      childDateOfBirth: ['', Validators.required],
      conditionDiagnosis: ['']
    });

    this.addOtherCaregiversForm = this.fb.group({
      firstName: ['', Validators.required],
      lastName: ['', Validators.required],
      email: ['',
        Validators.compose([Validators.maxLength(70),
          Validators.email,
          Validators.required])]
    });

    this.addTherapyProvider = this.fb.group({
      therapyOrganizationName: ['', Validators.required],
      city: ['', Validators.required],
      therapistFirstName: ['', Validators.required],
      therapistLastName: ['', Validators.required],
      therapistEmail: ['',
        Validators.compose([Validators.maxLength(70),
          Validators.email,
          Validators.required])]
    });

    this.addAdditionalOrganizations = this.fb.group({
      therapyOrganizationName: ['', Validators.required],
      city: ['', Validators.required],
      therapistFirstName: ['', Validators.required],
      therapistLastName: ['', Validators.required],
      therapistEmail: ['',
        Validators.compose([Validators.maxLength(70),
          Validators.email,
          Validators.required])]
    });
  }

}
