import { Component, OnInit, NgZone } from '@angular/core';
import * as moment from 'moment';
import { Observable } from 'rxjs';
import { WorkoutService } from 'src/app/services/workout.service';
import { Unsubscriber } from '../common/unsubscriber';
import { takeUntil, switchMap, tap } from 'rxjs/operators';
import { ActivatedRoute, Params } from '@angular/router';

import { Plugins } from '@capacitor/core';
import { StateService } from 'src/app/services/state.service';
import { SessionService } from 'src/app/services/session.service';
import { Platform, ModalController, ToastController } from '@ionic/angular';
import { SessionViewAttachmentComponent } from '../session/session-view-attachment/session-view-attachment.component';
import { PhotoViewer } from '@ionic-native/photo-viewer/ngx';
import { UIFileState } from 'src/app/models/interfaces/ui-file-state';
import { ToastService } from 'src/app/services/toast.service';
import { FileService } from 'src/app/services/file.service';
import { LoadingContentComponent } from '../session/loading-content/loading-content.component';
import { WebVideoComponent } from '../session/web-video/web-video.component';

const { CapacitorVideoPlayer, Device } = Plugins;

@Component({
  selector: 'app-workouts',
  templateUrl: './workouts.component.html',
  styleUrls: ['./workouts.component.scss'],
})
export class WorkoutsComponent extends Unsubscriber implements OnInit {
  pageTitle = 'Daily Focus';
  currentDate = moment().format('MMMM YYYY');
  workout$: Observable<any>;
  private patientId;
  private sessionId;
  public workoutCompleted = false;
  public slideOpts = {
    initialSlide: 0,
    speed: 400,
    centeredSlides: false,
    slidesPerView: 'auto',
    freeMode: true
  };

  constructor(
    private workoutService: WorkoutService,
    private route: ActivatedRoute,
    public stateService: StateService,
    private sessionService: SessionService,
    private modalCtrl: ModalController,
    private photoViewer: PhotoViewer,
    public plt: Platform,
    public toast: ToastService,
    public ngZone: NgZone,
    public fileService: FileService
  ) {
    super();
  }

  ngOnInit() {
    this.workout$ = this.route.paramMap.pipe(
      switchMap((params: Params) => {
        this.patientId = params.get('patient_id');
        return this.workoutService.getDailyWorkout(this.patientId).pipe(
          tap((res) => {
            this.sessionId = res.sessionId;
            this.workoutCompleted = res.isComplete;
          }));
      }));
  }

  viewDocument(document) {
    console.log('opening document');
    console.log(JSON.stringify(document));
    if (this.stateService.getPlatform() === 'web') {
      this.downloadFile(document, false, true);
    } else {
      console.log('DOCUMENT TO OPEN: ' + JSON.stringify(document));
      if (document.state === UIFileState.Exists) {
        return this.openFile(document);
      }
      this.downloadFile(document);
    }
  }

  private downloadFile(document, autoOpen = true, web = false) {
    console.log('DOWNLOADING FILE..');
    this.ngZone.run(() => {
      document.state = UIFileState.Downloading;
    });

    // this.sessionService.getDocumentDownloadUrl(document.id).pipe(takeUntil(this._unsubscriber))
    //   .subscribe((result) => {
    if (this.stateService.getPlatform() === 'web') {
      window.open(document.file_url, '_blank');
    } else {
      this.fileService.downloadUrl(document.file_url, document.file.filename, document.id, { autoOpen });
    }
      // });
  }

  public async openFile(document: any) {
    console.log('Opening file...');
    try {
      await this.fileService.openDownloadedFile(document.id, document.mime_type);
    } catch (e) {
      console.error(e);
      this.toast.presentToast('Failed to open file.');
    }
  }

  async playVideo(video) {
    const modal = await this.modalCtrl.create({
        component: LoadingContentComponent
      });
    await modal.present();
    this.sessionService.getSessionVideoPlayUrl(this.sessionId, video.id).pipe(
      takeUntil(this._unsubscriber)
    ).subscribe(async (videoUrl) => {
      if (this.stateService.getPlatform() === 'ios' || this.stateService.getPlatform() === 'android') {
        const videoPlayer = CapacitorVideoPlayer;
        this.isPlayingVideo(video.file_url, videoPlayer, modal);
      } else {
        modal.dismiss();
        const webVideoModal = await this.modalCtrl.create({
          component: WebVideoComponent,
          componentProps: {
            video
          }
        });
        await webVideoModal.present();
      }
    });
  }

  isPlayingVideo(video, videoPlayer, modal) {
    setTimeout(async () => {
      modal.dismiss();
      await videoPlayer.initPlayer({mode: 'fullscreen', url: video, playerId: 'player-id', componentTag: 'app-session'});
    }, 1000);
  }

  completeWorkout() {
    this.workoutService.completeWorkout(this.patientId, this.sessionId).pipe(takeUntil(this._unsubscriber)).subscribe((val: any) => {
      this.workoutService.setDailyWorkout(val);
      this.workoutCompleted = this.workoutCompleted ? false : true;
    });
  }

  async imageClicked(url) {
    if (this.plt.is('cordova')) {
      this.photoViewer.show(url);
    } else {
      const modal = await this.modalCtrl.create({
        component: SessionViewAttachmentComponent,
        componentProps: {
          link: url
        }
      });
      await modal.present();
    }
  }

}
