import { Injectable, NgZone } from '@angular/core';
import { Observable, Observer } from 'rxjs';
import { User } from '../models/client/User';
import * as auth0 from 'auth0-js';
import { TokenService } from './token.service';
import { UserService } from './user.service';
import { SocketsService } from './sockets.service';
import { StorageService } from './storage.service';
import { JwtHelperService } from '@auth0/angular-jwt';
import { environment } from 'src/environments/environment';
import { Router } from '@angular/router';

@Injectable({
  providedIn: 'root'
})
export class AuthService {
  webAuth = new auth0.WebAuth({
    clientID: environment.AUTH0_API_NATIVE_CLIENT_ID,
    domain: environment.AUTH0_API_DOMAIN,
    audience: environment.AUTH0_JWKS_AUDIENCE,
    scope: environment.AUTH0_API_SCOPE
  });
  jwtHelper = new JwtHelperService();

  constructor(
    private tokenService: TokenService,
    private userService: UserService,
    private storageService: StorageService,
    private socketsService: SocketsService
    ) {
    this.tokenService.checkForToken();
  }

  public login(credentials): Observable<any> {
    return new Observable((subscriber) => {
      this.webAuth.client.login({
        realm: environment.AUTH0_API_REALM,
        username: credentials.email,
        password: credentials.password,
        scope: environment.AUTH0_API_SCOPE,
        audience: environment.AUTH0_JWKS_AUDIENCE
      }, (err, authResult) => {
        if (err) {
          this.handleAuthError(err, subscriber);
        } else {
          this.handleAuthSuccess(authResult, subscriber);
        }
      });
    });
  }

  public autoLogin(): Observable<any> {
    return new Observable((subscriber) => {
      const storedToken = this.tokenService.token;
      if (storedToken) {
        this.handleAuthSuccess({ accessToken: storedToken }, subscriber);
      } else {
        this.handleAuthError({ error: new Error('no token found') }, subscriber);
      }
    });
  }

  private handleAuthError(error: auth0.Auth0Error, subscriber: Observer<any>) {
    subscriber.error(error);
    subscriber.complete();
  }

  private async handleAuthSuccess(authResult: any, subscriber: Observer<any>) {
    await this.tokenService.storeToken(authResult.accessToken);
    const decodedToken = this.tokenService.decodeJwt(this.tokenService.token);
    const auth0UserId: string = decodedToken.sub;

    this.userService.getUserByAuth0Id(auth0UserId).subscribe(
      (res) => {
      this.userService.user = new User(res, authResult.accessToken);
      this.userService.setPreviousUser({
        first_name: this.userService.user.first_name,
        last_name: this.userService.user.last_name,
        email: this.userService.user.email
      });
      this.socketsService.connect();
      subscriber.next(authResult);
      subscriber.complete();
    },
      (err) => {
      subscriber.error(err);
    });
  }

  public async invalidate() {
    await this.tokenService.deleteToken();
    await this.storageService.deleteObject('previousUser');
    this.userService.previousUser.next(null);
    this.userService.user = null;
  }

  public isAuthenticated() {
    return this.tokenService.token ? !this.jwtHelper.isTokenExpired(this.tokenService.token) : false;
  }
}
