import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { environment } from 'src/environments/environment';
import { IDocument } from '../models/server/Document';
import { Observable, BehaviorSubject, Subject } from 'rxjs';
import { IHistory } from '../models/server/history';
import { FileService, FileUpload, FileUploadState } from '../services/file.service';
import { v4 } from 'uuid';
import * as _ from 'lodash';
import { PatientService } from './patient.service';

export interface IDocumentsResponse {
  documents: IDocument[];
}
export interface DocumentUpload extends FileUpload {
  documentId?: string;
}

export enum DocumentUploadType {
  Document,
  InvisibleDocument
}


@Injectable({
  providedIn: 'root'
})
export class DocumentService {
  public readonly documentUploadStore: DocumentUpload[] = [];
  public uploadChange$: Observable<DocumentUpload>;
  private uploadChangeSrc: Subject<DocumentUpload>;
  public uploadSuccess: Subject<DocumentUpload> = new Subject<DocumentUpload>();

  public documents: BehaviorSubject<IDocument[]> = new BehaviorSubject<IDocument[]>([]);
  constructor(
    private http: HttpClient,
    private fileService: FileService,
    private patientService: PatientService
  ) {
    this.uploadChangeSrc = new Subject<DocumentUpload>();
    this.uploadChange$ = this.uploadChangeSrc.asObservable();

    fileService.uploadStarted$.subscribe((upload) => {
      if (!this.isDocumentUpload(upload)) {
        return;
      }
      this.updateDocumentUpload(upload);

      this.uploadChangeSrc.next(upload);
    });

    fileService.uploadProgress$.subscribe((upload) => {
      if (!this.isDocumentUpload(upload)) {
        return;
      }
      this.updateDocumentUpload(upload);

      this.uploadChangeSrc.next(upload);
    });

    fileService.uploadFinish$.subscribe((upload) => {
      if (!this.isDocumentUpload(upload)) {
        return;
      }
      this.finalizeDoc(upload);
      this.removeStoredUpload(upload);
    });

    fileService.uploadError$.subscribe((upload) => {
      if (!this.isDocumentUpload(upload)) {
        return;
      }

      console.error('D upload error', upload);

      this.removeStoredUpload(upload);
      this.uploadChangeSrc.next(upload);
    });
  }

  public isDocumentUpload(upload: FileUpload) {
    switch (upload.meta.type) {
      case DocumentUploadType.Document:
      case DocumentUploadType.InvisibleDocument:
        return true;
      default:
        return false;
    }
  }

  public getDocumentUploadIndex(upload: FileUpload) {
    return _.findIndex(this.documentUploadStore, d => d.meta.tmpKey === upload.meta.tmpKey);
  }


  public updateDocumentUpload(upload: FileUpload) {
    const idx = this.getDocumentUploadIndex(upload);
    if (idx === -1) {
      return;
    }
    this.documentUploadStore[idx] = Object.assign({}, upload) as DocumentUpload;
  }

  public removeStoredUpload(upload: FileUpload) {
    _.remove(this.documentUploadStore, (d) => d.meta.tmpKey === upload.meta.tmpKey);
  }

  getDocuments() {
    const patientId = this.patientService.currentPatient.value.id;
    let url = `${environment.API_URL}/api/documents`;
    if (patientId) {
      url += `?patientId=${patientId}`;
    }

    this.http.get<IDocumentsResponse>(url).subscribe((res) => {
      this.documents.next(res.documents);
    });
  }

  public createDocument(fileId: string, patientId?: string, visible?: boolean) {
    const request = {
      fileId,
      patientId,
      visible
    };
    return this.http.post(`${environment.API_URL}/api/documents`, request);
  }

  updateDocument(documentId: string, document: IDocument) {
    return this.http.put<IDocument>(`${environment.API_URL}/api/documents/${documentId}`, document);
  }

  deleteDocument(documentId: string) {
    return this.http.delete(`${environment.API_URL}/api/documents/${documentId}`);
  }

  public uploadDocument(file: any, patientId: string, meta?: any): DocumentUpload {
    const procKey = v4();
    const combinedMeta = Object.assign({
      type: DocumentUploadType.Document,
      tmpKey: procKey,
      patientId
    }, meta);

    const fileUpload = this.fileService.uploadFileAsync(file, combinedMeta);

    this.documentUploadStore.push(Object.assign({}, fileUpload) as DocumentUpload);

    this.uploadChangeSrc.next(fileUpload);

    return fileUpload;
  }

  private finalizeDoc(upload: FileUpload) {
    const type = upload.meta.type;
    const patientId = upload.meta.patientId;
    const fileId = upload.key;

    const sub = this.createDocument(fileId, patientId, type !== DocumentUploadType.InvisibleDocument)
      .subscribe((response: any) => {
        sub.unsubscribe();
        const docUpload: DocumentUpload = Object.assign({ documentId: response.document.id }, upload);
        docUpload.state = FileUploadState.Success;
        this.uploadChangeSrc.next(docUpload);
        this.uploadSuccess.next(upload);
        this.getDocuments();
      }, (e) => {
        sub.unsubscribe();
        console.error('finalizeInvisDoc', e);

        upload.state = FileUploadState.Error;
        this.uploadChangeSrc.next(upload as DocumentUpload);
      });
  }
}
