import { Injectable, NgZone } from '@angular/core';
import { Camera as CordovaCamera } from '@ionic-native/camera/ngx';
import { DomSanitizer } from '@angular/platform-browser';
import { SessionService } from './session.service';
import { takeUntil, map } from 'rxjs/operators';
import { FileTransferService } from './file-transfer.service';
import { Plugins } from '@capacitor/core';
import { forkJoin, Subject } from 'rxjs';
import { Unsubscriber } from '../components/common/unsubscriber';
import { ToastService } from 'src/app/services/toast.service';
import { StateService } from 'src/app/services/state.service';
import { FileService } from './file.service';
const { Device } = Plugins;

export enum UploadType {
  CommentAttachment = 'commentAttachment',
  SessionAttachment = 'sessionAttachment',
  Document = 'document',
  WebSessionAttachment = 'webSessionAttachment'
}

export interface IHttpRequestCreateCommentAttachment {
  comment_id: string;
  video_file_id: string;
  thumbnail_file_id: string;
}

@Injectable({
  providedIn: 'root'
})
export class MediaService extends Unsubscriber {

  uploadAttachmentObj: Subject<any> = new Subject<any>();

  constructor(
    private ngZone: NgZone,
    private toast: ToastService,
    private cordovaCam: CordovaCamera,
    private sanitizer: DomSanitizer,
    private sessionService: SessionService,
    private fileService: FileService,
    private fileTransferService: FileTransferService,
    private stateService: StateService
  ) {
    super();
   }

  async selectFromPhotoLibrary() {
    const info = await Device.getInfo();
    let mediaFile;
    if (this.stateService.getPlatform() === 'ios') {
      const selectedMedia = await this.cordovaCam.getPicture({
        sourceType: this.cordovaCam.PictureSourceType.PHOTOLIBRARY,
        destinationType: this.cordovaCam.DestinationType.FILE_URI,
        mediaType: this.cordovaCam.MediaType.ALLMEDIA
      }).then((fileURI) => {
        return fileURI.indexOf('file://') === -1 ? `file://${fileURI}` : fileURI;
      });
      const name = selectedMedia.substr(selectedMedia.lastIndexOf('/') + 1);
      mediaFile = {
        path: selectedMedia,
        thumbnailPath: selectedMedia,
        safeThumbnailPath: this.sanitizer.bypassSecurityTrustResourceUrl(selectedMedia),
        name,
        tags: [],
        category: 'Default',
      };
    } else if (this.stateService.getPlatform() === 'android') {
      // todo: kjm - is this the best way to do this on android?
        const pickedFile = await this.fileService.filePicker.pickFile();
        const selectedMedia = pickedFile;
        const name = selectedMedia.substr(selectedMedia.lastIndexOf('/') + 1);
        mediaFile = {
          path: selectedMedia,
          thumbnailPath: selectedMedia,
          safeThumbnailPath: this.sanitizer.bypassSecurityTrustResourceUrl(selectedMedia),
          name,
          tags: [],
          category: 'Default',
        };
      } else {
      // todo:kjm - web implementation
    }
    return mediaFile;
  }

  uploadMedia(mediaFile, uploadType: UploadType, sessionId: string = null) {
    const requestFileUploadPostUrl = this.sessionService.requestFileUploadPostUrl(mediaFile.path, 0, 0);
    forkJoin([requestFileUploadPostUrl])
    .pipe(
      map((s3response) => {
        return this.formatUploadObject(s3response, uploadType, mediaFile, sessionId);
      }))
    .pipe(
      takeUntil(this._unsubscriber)).
        subscribe((uploadObject) => {
          const uploadId = this.fileTransferService.uploadFile(uploadObject.video, this.ngZone);
          uploadObject.attachmentObject.uploadId = uploadId;
          this.uploadAttachmentObj.next(uploadObject);
          // todo: kjm - let caller know this process has begun
        }, () => {
          this.toast.presentToast(
            'There was a problem uploading media, please try again. If you are uploading a long video, we recommend connecting to WiFi.');
        });
  }

  formatUploadObject(uploadUrlResponse, uploadType: UploadType, attachment, sessionId) {
    const fileS3 = uploadUrlResponse[0];
    let attachmentObject;

    switch (uploadType) {
      case UploadType.CommentAttachment:
        attachmentObject = {
          video_file_id: fileS3.file.id,
          thumbnail_file_id: null,
          uploadId: null
        };
        break;
      case UploadType.SessionAttachment:
        attachmentObject = {
          video_file_id: fileS3.file.id,
          thumbnail_file_id: null,
          session_id: sessionId,
          session_sub_type: attachment.category,
          tags: attachment.tags,
          uploadId: null
        };
        break;
      case UploadType.Document:
        // todo: kjm - map attachment object for document upload
        break;
    }
    return {
      video: {
        localFilePath: attachment.path,
        uploadUrl: fileS3.uploadInfo.url,
        fileName: fileS3.file.filename,
        mimeType: fileS3.file.mime_type,
        fields: fileS3.uploadInfo.fields,
      },
      thumb: null,
      attachmentObject
    };
  }
}

