import { Injectable, NgZone } from '@angular/core';
import { Plugins, PushNotification, PushNotificationToken, PushNotificationActionPerformed } from '@capacitor/core';
import { Router } from '@angular/router';
import { HttpClient } from '@angular/common/http';
import { BehaviorSubject } from 'rxjs';
import { UserService } from './user.service';
import { StateService } from './state.service';
import { AuthService } from './auth.service';
import { environment } from 'src/environments/environment';
import { PushNotificationType } from '../enums/PushNotificationType';
import { CommunicationService } from './communication.service';
import { INotification } from '../models/server/Notification';
import { ToastService } from './toast.service';
const { PushNotifications } = Plugins;

declare var VoIPPushNotification: any;
// declare var WebNotification: any;
declare let cordova: any;

@Injectable({
    providedIn: 'root',
})
export class NotificationService {
    public notificationCount: BehaviorSubject<number> = new BehaviorSubject<number>(0);
    public notifications: BehaviorSubject<INotification[]> = new BehaviorSubject([]);
    public chatMessageCount: BehaviorSubject<number> = new BehaviorSubject<number>(0);
    public chatMessages: BehaviorSubject<INotification[]> = new BehaviorSubject([]);

    constructor(
        private authService: AuthService,
        private userService: UserService,
        private stateService: StateService,
        private communicationService: CommunicationService,
        private router: Router,
        private ngZone: NgZone,
        private http: HttpClient,
        private toastService: ToastService
    ) {}

    registerNotifications() {
        this.loadUserNotifications();

        if (this.stateService.getPlatform() === 'web') {
            // Don't try to register push notifications on web
            return;
        }

        // On success, we should be able to receive notifications
        PushNotifications.addListener('registration', (token: PushNotificationToken) => {
            console.log('Push registration success: ' + token.value);
            if (!!token && !!token.value && this.authService.isAuthenticated()) {
                this.userService.registerDevice(token.value).subscribe((result: any) => {
                    console.log('Device Registered: ', JSON.stringify(result));
                });
            }
        });

        // Some issue with our setup and push will not work
        PushNotifications.addListener('registrationError', (error: any) => {
            console.log('Error on registration: ' + JSON.stringify(error));
        });

        // Show us the notification payload if the app is open on our device
        PushNotifications.addListener('pushNotificationReceived', (notification: PushNotification) => {
            console.log('Push received: ' + JSON.stringify(notification));
            const notificationData = this.getNotificationData(notification);
            if (!!notificationData) {
                this.handleReceivedNotification(notificationData);
            }
            this.loadUserNotifications();
        });

        // Method called when tapping on a notification
        PushNotifications.addListener(
            'pushNotificationActionPerformed',
            (notificationAction: PushNotificationActionPerformed) => {
                console.log('Push action performed: ' + JSON.stringify(notificationAction));
                const notification = this.getNotificationData(notificationAction.notification);
                if (!notification) {
                    console.log('Could not process empty notification since it was null');
                    return;
                }
                this.handleTappedNotification(notification);
            }
        );

        // Register with Apple / Google to receive push via APNS/FCM
        PushNotifications.requestPermission().then( result => {
            if (result.granted) {
              // Register with Apple / Google to receive push via APNS/FCM
              PushNotifications.register().then((response) => {
                  console.log('[NOTIFICATIONS] register complete: ', response);
                }, (err) => {
                  console.error('[NOTIFICATIONS] register error: ', err);
                });
            } else {
              // Show some error
              this.toastService.presentToastTopWithClose('Please enable notifications to take advantage of all features including video calling.', 5000);
            }
          });

        // This will only run for iOS devices
        const voipNotification = VoIPPushNotification.init();

        voipNotification.on('registration', (data) => {
            console.log(`VoIP registration success: ${data.deviceToken}`);
            if (!!data && !!data.deviceToken && this.authService.isAuthenticated()) {
                this.userService.registerVoipToken(data.deviceToken).subscribe((result: any) => {
                    console.log('VoIP Registered: ', JSON.stringify(result));
                });
            }
        });

        voipNotification.on('notification', (data) => {
            console.log('VoIP notification received');
            console.log(JSON.stringify(data));

            this.communicationService.setIncomingCall({
                token: data.extra.token,
                roomName: data.extra.roomName,
            });
        });

        voipNotification.on('error', (e) => {
            console.log(JSON.stringify(e));
        });

        const cordovaCall = (cordova.plugins as any).CordovaCall;

        cordovaCall.setVideo(true);

        cordovaCall.on('answer', () => {
            console.log('Call Answered');
            // Show video call
            this.ngZone.run(() => this.router.navigate([`/video`])).then();
        });

        cordovaCall.on('hangup', () => {
            console.log('Call Hangup');
            // Send hangup info to backend
        });

        cordovaCall.on('reject', () => {
            console.log('Call Reject');
            // Send call rejection to backend
        });

        cordovaCall.on('receiveCall', () => {
            console.log('Receive Call');
        });
    }

    getNotificationData(notification: PushNotification): any {
        try {
            if (this.stateService.getPlatform() === 'ios') {
                return notification.data.data;
            } else if (this.stateService.getPlatform() === 'android') {
                return JSON.parse(notification.data.extra);
            }
            return null;
        } catch (e) {
            return null;
        }
    }

    handleTappedNotification(notification) {
        const notificationType = notification.type;
        switch (notificationType) {
            case PushNotificationType.Session:
            case PushNotificationType.Comment:
                const sessionId: string = notification.sessionId;
                this.ngZone.run(() => this.router.navigate([`/patient/session/${sessionId}`])).then();
                break;
        }
    }

    handleReceivedNotification(notification) {
        const notificationType = notification.type;
        switch (notificationType) {
            case PushNotificationType.VideoCallIncoming:
                if (this.stateService.getPlatform() === 'android') {
                    this.communicationService.setIncomingCall({
                        token: notification.token,
                        roomName: notification.roomName,
                    });
                }
                break;
        }
    }

    public loadUserNotifications() {
        this.getUserNotifications().subscribe(val => {
            const notifications = [];
            let notificationCount = 0;

            const chatMessages = [];
            let chatMessageCount = 0;

            val.notifications.forEach((notification) => {
               if (notification.type === 'CHAT_MESSAGE') {
                   chatMessageCount++;
                   chatMessages.push(notification);
               } else {
                   notificationCount++;
                   notifications.push(notification);
               }
            });

            this.notifications.next(notifications);
            this.notificationCount.next(notificationCount);
            this.chatMessages.next(chatMessages);
            this.chatMessageCount.next(chatMessageCount);
        });
    }

    public deleteSingleNotification(notificationId: string, notificationType?: string) {
        this.deleteSingleNotificationFromServer(notificationId).subscribe(async () => {
            const n = await this.notifications.getValue();
            if (notificationType === 'CHAT_MESSAGE') {
                this.chatMessages.next(n.filter(item => item.id !== notificationId));
                this.chatMessageCount.next(this.chatMessages.value.length);
            } else {
                this.notifications.next(n.filter(item => item.id !== notificationId));
                this.notificationCount.next(this.notifications.value.length);
            }
        });
    }

    public deleteAllNotifications() {
        this.deleteAllNotificationsFromServer().subscribe(async () => {
            this.notifications.next([]);
            this.notificationCount.next(0);
          });
    }

    // returns a list of notifications for logged in user
    private getUserNotifications() {
        return this.http.get<any>(`${environment.API_URL}/api/users/notifications`);
    }

    private deleteAllNotificationsFromServer() {
        return this.http.delete(`${environment.API_URL}/api/users/notifications`);
    }

    private deleteSingleNotificationFromServer(notificationId) {
        return this.http.delete(`${environment.API_URL}/api/users/notifications/${notificationId}`);
    }

}
