import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { environment } from 'src/environments/environment';
import { Observable, BehaviorSubject } from 'rxjs';
import { GetSessionResponse } from '../models/http/GetSessionResponse';
import { SessionAttachmentResponse } from '../models/http/SessionAttachmentResponse';
import { SessionAttachment } from '../models/client/SessionAttachment';
import { TokenService } from './token.service';
import * as _ from 'lodash';
import { GetSpecialtiesResponse, Session } from '../models/Server';
import { SessionVideoUpload } from '../models/client/SessionVideoUpload';
import { CreateSessionAttachmentRequest } from '../models/server/SessionAttachment';
import { GetCommentResponse } from '../models/http/GetCommentResponse';

export enum FileState {
  complete = 'COMPLETE',
  pending = 'PENDING'
}

export interface FileModel {
  id: string;
  path: string;
  filename: string;
  original_filename?: string;
  created_by_user_id: string;
  state: FileState;
  mime_type?: string;
  created_at: string;
  updated_at: string;
  duration?: number;
  file_size?: number;
}

export interface FileUploadUrlResult {
  url: string;
  mime: string;
  file: FileModel;
  uploadInfo: any;
}

export interface ISessionComment {
  session_id: string;
  content: string;
}

@Injectable({
  providedIn: 'root'
})
export class SessionService {
  BATCH_ID = 1;
  public sessionState$: BehaviorSubject<any>;

  constructor(private http: HttpClient, private tokenService: TokenService) { }

  public getSessions(patientId) {
    return this.http.get(`${environment.API_URL}/api/patients/${patientId}/sessions`);
  }

  public getSessionData(sessionId: string): Observable<GetSessionResponse> {
    return this.http.get<GetSessionResponse>(`${environment.API_URL}/api/sessions/${sessionId}`);
  }

  public deleteSession(sessionId: string) {
    return this.http.delete(`${environment.API_URL}/api/sessions/${sessionId}`);
  }

  public getSessionAttachment(sessionId: string, attachmentId: string): Observable<SessionAttachmentResponse> {
    return this.http.get<SessionAttachmentResponse>(`${environment.API_URL}/api/sessions/${sessionId}/attachment/${attachmentId}`);
  }

  public createSession(patientId: string): Observable<any> {
    return this.http.post(`${environment.API_URL}/api/sessions`, { patientId });
  }

  public publishSession(sessionId: string) {
    return this.http.post(`${environment.API_URL}/api/sessions/${sessionId}/publish`, null);
  }

  public updateSession(session: Session): Observable<GetSessionResponse> {
    return this.http.put(`${environment.API_URL}/api/sessions/${session.id}`, session);
  }

  public convertSessionVideos(sessionId: string, sessionVideos: SessionAttachmentResponse[]): SessionAttachment[] {
    return _.map(sessionVideos, (sv) => this.convertSessionVideo(sessionId, sv));
  }

  public convertSessionVideo(sessionId: string, sessionVideo: SessionAttachmentResponse): SessionAttachment {
    const newSessionAttachment = {
      id: sessionVideo.id,
      category: sessionVideo.session_sub_type,
      isSelected: false,
      thumbnailPath: this.getThumbUrl(sessionVideo),
      tags: sessionVideo.tags,
      sessionId: sessionVideo.session_id,
      path: sessionVideo.file_url
    };
    console.log('new session attachment', newSessionAttachment);
    this.getSessionVideoPlayUrl(sessionId, sessionVideo.id).subscribe((val) => {
      console.log('video url', val);
    });
    return newSessionAttachment;
  }

  public getThumbUrl(video: SessionAttachmentResponse) {
    const url = environment.API_URL + `/api/sessions/${video.session_id}/video/`;
    return url + video.id + `/thumb?twtoken=${this.tokenService.token}`;
  }

  public getSpecialties(): Observable<GetSpecialtiesResponse> {
    return this.http.get<GetSpecialtiesResponse>(`${environment.API_URL}/api/specialties`);
  }

  public createSessionAttachment(sessionId: string, attachment: CreateSessionAttachmentRequest) {
    return this.http.post<SessionAttachment>(`${environment.API_URL}/api/sessions/${sessionId}/attachment`, attachment);
  }

  public requestFileUploadUrl(fileName: string, duration: number = 0, fileSize: number = 0) {
    console.log('REQUESTING URL', fileName, duration, fileSize);
    return this.http.post<FileUploadUrlResult>(`${environment.API_URL}/api/files`, { fileName, duration, fileSize });
  }

  public requestFileUploadPostUrl(fileName: string, duration: number = 0, fileSize: number = 0) {
    console.log('REQUESTING URL', fileName, duration, fileSize);
    return this.http.post<FileUploadUrlResult>(`${environment.API_URL}/api/files/postfile`, { fileName, duration, fileSize });
  }

  public getFileUploadUrl(fileName: string, duration: number, fileSize: number) {
    return this.requestFileUploadUrl(fileName, duration, fileSize).toPromise();
  }

  public uploadFile(file: any) {
    return this.http.post(`${environment.API_URL}/api/files`, { data: file });
  }

  public getSessionVideoPlayUrl(sessionId: string, sessionVideoId: string) {
    return this.http.get(`${environment.API_URL}/api/sessions/${sessionId}/video/${sessionVideoId}/play`);
  }

  public getSessionCommentVideoPlayUrl(sessionId: string, commentId: string, commentVideoId: string) {
    return this.http.get(`${environment.API_URL}/api/sessions/${sessionId}/comment/${commentId}/video/${commentVideoId}/play`);
  }

  public getSessionComments(sessionId: string): Observable<GetCommentResponse> {
    return this.http.get<GetCommentResponse>(`${environment.API_URL}/api/sessions/${sessionId}/comments`);
  }

  public postSessionComment(commentData): Observable<GetCommentResponse> {
    return this.http.post<GetCommentResponse>(`${environment.API_URL}/api/sessions/${commentData.session_id}/comment`, commentData);
  }

  public postSessionReaction(sessionId: string) {
    const reactionData = {
      session_id: sessionId,
      reaction_type: 'LOVE'
    };
    return this.http.post<any>(`${environment.API_URL}/api/sessions/${sessionId}/reaction`, reactionData);
  }
}
